

# This is an R-function to give summary statistics of Extremal QR

summary.rq.extreme<- function (object, subsample.fraction=.2, R=100, method = "fn", alpha = .10, spacing = 20, ...)
{
  mt 		<- terms(object)
  m 		<- model.frame(object)
  y 		<- model.response(m)
  x 		<- model.matrix(mt, m, contrasts = object$contrasts)
  wt 		<- model.weights(m)
  tau 	<- object$tau
  eps 	<- .Machine$double.eps^(2/3)
  coef 	<- coefficients(object)
  if (is.matrix(coef)) coef <- coef[, 1]
  vnames <- dimnames(x)[[2]]
  resid <- object$residuals
  n 		<- length(resid)
  p	 	  <- length(coef)
  rdf 	<- n - p
  if (!is.null(wt))
  {
   	resid <- resid * wt
   	x 	  <- x * wt
   	y 	  <- y * wt
 	}

  infer.object 	<- rq.infer.extreme(x, y, tau = tau, subsample.fraction=subsample.fraction, spacing = spacing, R=R, method = method, ...)

  coef <- array(coef, c(p, 5))
  dimnames(coef) <- list(vnames, c("Value", "Psedo Std. Er.", "Lower  5% Bound", "Upper 95% Bound", "Bias-Corrected Estimate"))
  coef[, 3] <- infer.object$ciL.e
  coef[, 4] <- infer.object$ciU.e
  coef[, 2] <- (coef[, 4]- coef[, 3]) / (2 * qnorm(1 - alpha / 2))
  coef[, 5] <- infer.object$BC.betat
  object 		<- object[c("call", "terms")]
  object$R	<-  infer.object$R
  object$coefficients	<- coef
  object$tau <- tau
  class(object) <- "summary.rq"
  object
}



# function that does not complain about singular designs and records instead a NaN

rq.fit.fn.nostop <- function (x, y, tau = 0.5, beta = 0.99995, eps = 1e-06)
{
  n <- length(y)
  p <- ncol(x)
  if (n != nrow(x))
  	stop("x and y don't match n")
  if (tau < eps || tau > 1 - eps)
    stop("tau is outside (0,1)")
  rhs <- (1 - tau) * apply(x, 2, sum)
  d 	<- rep(1, n)
  u   <- rep(1, n)
  wn  <- rep(0, 10 * n)
  wn[1:n] <- (1 - tau)
  z <- .Fortran("rqfn", as.integer(n), as.integer(p), a = as.double(t(as.matrix(x))),
      		c = as.double(-y), rhs = as.double(rhs), d = as.double(d),
      		as.double(u), beta = as.double(beta), eps = as.double(eps),
      		wn = as.double(wn), wp = double((p + 3) * p), aa = double(p * p),
      		it.count = integer(2), info = integer(1), PACKAGE = "quantreg")
  if (z$info != 0)
  {
  	coefficients <- NA
	}
	else
	{
		coefficients <- -z$wp[1:p]
	}
  list(coefficients = coefficients)
}



# This function computes Extremal QR

rq.infer.extreme <- function(X,Y, tau=.1, subsample.fraction=.2, spacing=20, R=100, method = "fn", alpha = .10, ...)
{
	if (tau>.5)
	{
		Y	<- -Y;
		tau.e	<- 1-tau
	}
	else
	  tau.e	<- tau

	n	<- length(Y);
	b	<- floor(subsample.fraction * n);
	m	<- (tau.e * n + spacing) / (tau.e * n);
	p <- dim(X)[2]
	tau.b.e	<- min(tau.e * n / b, tau.e + (.5 - tau.e) / 3);
  if (tau.b.e == tau.e + (.5 - tau.e) / 3 && b >= min(n / 3, 1000))
		warning("tau may be non-extremal; results are not likely to differ from central inference");
 	muX		<- apply(X,2, mean);
	betat	<- rq(Y~X[,-1], tau=tau.e, method= method)$coef;
	betatm<- rq(Y~X[,-1], tau=m*tau.e, method= method)$coef;
	An		<- (m-1)*tau.e*sqrt(n/(tau.e*(1-tau.e)) )/(muX%*%(  betatm - betat))
	betatb.e <- rq(Y~X[,-1], tau=tau.b.e, method=method)$coef;

	Res		<- NULL;
	ss		<- matrix(sample(1:n, b * R, replace = T), b, R);
  m.b.e	<- (tau.b.e*b+spacing)/(tau.b.e*b);

  for(i in 1:R)
	{
		Xs <- X[ss[,i],];
		Ys <- Y[ss[,i]];
		if (method == "fn")
		{
			sub.betatb.e	<- rq.fit.fn.nostop(Xs,Ys, tau=tau.b.e)$coef;
			sub.betatbm.e	<- rq.fit.fn.nostop(Xs, Ys, tau=m.b.e*tau.b.e)$coef;
		}
		else
		{
			sub.betatb.e	<- coef(rq(Ys ~ Xs[ ,-1], tau=tau.b.e));
			sub.betatbm.e	<- coef(rq(Ys ~ Xs[ ,-1], tau=m.b.e*tau.b.e));
		}
 		Ab.e <- (m.b.e-1)*tau.b.e*sqrt(b/(tau.b.e * (1-tau.b.e))) / (muX%*%(sub.betatbm.e - sub.betatb.e));
		Sb.e <- Ab.e * (sub.betatb.e -  betatb.e);
		Res	 <- rbind(Res, Sb.e)
  }


  object <- NULL
  names(betat) <- dimnames(X)[[2]]
  dimnames(Res)[[2]] <- dimnames(X)[[2]]
  if(tau <=.5)
  {
  	object$betat	<- betat
  	object$BC.betat	<- betat - apply(Res, 2, quantile, .5, na.rm=TRUE)/An;
  	object$ciL.e	<- betat - apply(Res, 2, quantile, 1 - alpha/2, na.rm=TRUE)/An;
  	object$ciU.e	<- betat - apply(Res, 2, quantile, alpha/2, na.rm=TRUE)/An;
	}
  else
  {
		object$betat	<-   -betat
  	object$BC.betat	<- -(betat - apply(Res, 2, quantile, .5, na.rm=TRUE)/An);
  	object$ciL.e	<- -(betat - apply(Res, 2, quantile, alpha/2, na.rm=TRUE)/An);
  	object$ciU.e	<- -(betat - apply(Res, 2, quantile, 1 - alpha/2, na.rm=TRUE)/An);
  }

  object$R <- R-sum(is.na(Res[,1]));
  return(object);
}



library(quantreg);




# Scale parameter estimator
scale.rq <- function (formula, tau=.1, m=2)
{
  beta.1 <- summary.rq(rq(formula, tau=tau), se="ker")$coefficients[,1]
  beta.m <- summary.rq(rq(formula, tau=m*tau), se="ker")$coefficients[,1]
  X.bar <- colMeans(model.matrix(formula))
  return ((beta.m-beta.1)/(X.bar %*% (beta.m-beta.1)))
}



# Hill estimator for EV index
hill.rq <- function (formula, tau=.1)
{
  beta <- summary.rq(rq(formula, tau=tau), se="ker")$coefficients[,1]
  y <- model.response(model.frame(formula))
  X <- model.matrix(formula)
  fitted <- X %*% beta
  z <- log(ifelse(y<=fitted, abs(y/fitted), 1))
  z <- z[is.finite(z)]
  return (sum(z)/(length(z)*tau))
}


# Inference on Hill estimator for EV index
infer.rq.hill <- function (formula, tau=.1, R=100, ci=.9, ...)
{
  if (tau > .5)
  {
    y <- -model.response(model.frame(formula))
    x <- -model.matrix(formula)
    formula <- y ~ x - 1
    tau <- 1 - tau
  }
  Res <- vector(length=R)
  xi <- hill.rq(formula, tau=tau)
  gamma <- scale.rq(formula, tau=tau)
  x <- model.matrix(formula)
  n <- dim(x)[1]
  scale <- x %*% gamma

  for (i in 1 : R)
  {
    ys <- (rexp(n)^{-xi}-1)/(-xi) * scale
    Res[i] <- hill.rq(ys ~ x - 1, tau=tau)
  }

  object <- NULL
  object$hill <- xi
  object$BC.hill <- xi - quantile(Res-xi, .5, na.rm=TRUE)
  object$ciL.e <- xi - quantile(Res-xi, (1+ci)/2, na.rm=TRUE)
  object$ciU.e <- xi - quantile(Res-xi, (1-ci)/2, na.rm=TRUE)
  object$se <- sqrt(var(Res))
  object$R <- R-sum(is.na(Res))
  return (object)
}

## Hill estimator for EV index
summary.rq.hill <- function (object, R=100, ci=.9, ...)
{
  mt <- terms(object)
  m <- model.frame(object)
  y <- model.response(m)
  x <- model.matrix(mt, m, contrasts = object$contrasts)
  wt <- model.weights(m)
  tau <- object$tau
  eps <- .Machine$double.eps^(2/3)
  coef <- coefficients(object)
  if (is.matrix(coef))
    coef <- coef[, 1]
  vnames <- dimnames(x)[[2]]
  resid <- object$residuals
  n <- length(resid)
  p <- length(coef)
  rdf <- n - p
  if (!is.null(wt))
  {
    resid <- resid * wt
    x <- x * wt
    y <- y * wt
  }

  res <- infer.rq.hill(y~x-1, tau=tau, R=R, ci=ci, ...)
  res <- cbind(res$hill, res$BC.hill, res$ciL.e, res$ciU.e)
  colnames(res) <- c("Estimate", "Bias-Corrected", paste("Lower ",50*(1-ci),"%",sep=""), paste("Upper ",50*(1+ci),"%",sep=""))
  rownames(res) <- paste("tau=", tau, sep="")
  return (res)
}




# Pickands estimator for EV index
pickands.rq <- function (formula, tau=.1, m=2)
{
  beta1 <- summary.rq(rq(formula, tau=tau), se="ker")$coefficients[,1]
  beta2 <- summary.rq(rq(formula, tau=m*tau), se="ker")$coefficients[,1]
  beta4 <- summary.rq(rq(formula, tau=2*m*tau), se="ker")$coefficients[,1]
  X.bar <- colMeans(model.matrix(formula))
  return ((-1/log(m)) * log((X.bar %*% (beta4-beta2))/(X.bar %*% (beta2-beta1))))
}

# Inference on Pickands estimator for EV index
infer.rq.pickands <- function (formula, tau=.1, R=100, ci=.9, ...)
{
  if (tau > .5)
  {
    y <- -model.response(model.frame(formula))
    x <- -model.matrix(formula)
    formula <- y ~ x - 1
    tau <- 1 - tau
  }
  Res <- vector(length=R)
  xi <- pickands.rq(formula, tau=tau)
  gamma <- scale.rq(formula, tau=tau)
  x <- model.matrix(formula)
  n <- dim(x)[1]
  scale <- x %*% gamma

  for (i in 1 : R)
  {
    ys <- (rexp(n)^{-xi}-1)/(-xi) * scale
    Res[i] <- pickands.rq(ys ~ x - 1, tau=tau)
  }

  object <- NULL
  object$pickands <- xi
  object$BC.pickands <- xi - quantile(Res-xi, .5, na.rm=TRUE)
  object$ciL.e <- xi - quantile(Res-xi, (1+ci)/2, na.rm=TRUE)
  object$ciU.e <- xi - quantile(Res-xi, (1-ci)/2, na.rm=TRUE)
  object$se <- sqrt(var(Res))
  object$R <- R-sum(is.na(Res))
  return (object)
}


## Pickands estimator for EV index
summary.rq.pickands <- function (object, R=100, ci=.9, ...)
{
  mt <- terms(object)
  m <- model.frame(object)
  y <- model.response(m)
  x <- model.matrix(mt, m, contrasts = object$contrasts)
  wt <- model.weights(m)
  tau <- object$tau
  eps <- .Machine$double.eps^(2/3)
  coef <- coefficients(object)
  if (is.matrix(coef))
    coef <- coef[, 1]
  vnames <- dimnames(x)[[2]]
  resid <- object$residuals
  n <- length(resid)
  p <- length(coef)
  rdf <- n - p
  if (!is.null(wt))
  {
    resid <- resid * wt
    x <- x * wt
    y <- y * wt
  }

  res <- infer.rq.pickands(y~x-1, tau=tau, R=R, ci=ci, ...)
  res <- cbind(res$pickands, res$BC.pickands, res$ciL.e, res$ciU.e)
  colnames(res) <- c("Estimate", "Bias-Corrected", paste("Lower ",50*(1-ci),"%",sep=""), paste("Upper ",50*(1+ci),"%",sep=""))
  rownames(res) <- paste("tau=", tau, sep="")
  return (res)
}




# Extrapolation estimator
# tau.e : extremal quantile of interest
# tau.s : stable quantile to use
# xi.hat : Pickands type estimator of EV index
extrap.rq <- function (formula, tau.e, tau.s, xi.hat=NULL, m=2)
{
  if (is.null(xi.hat))
  {
    xi.hat = pickands.rq(formula, tau=tau.s)
  }
  beta.1 <- summary.rq(rq(formula, tau=tau.s), se="ker")$coefficients[,1]
  beta.m <- summary.rq(rq(formula, tau=m*tau.s), se="ker")$coefficients[,1]
  return (((tau.e/tau.s)^(-xi.hat)-1)/(m^(-xi.hat)-1)*(beta.m-beta.1)+beta.1)
}
