
# this section implements the estimation and inference results for the VaR example


# clear workspace
rm(list = ls())


setwd("C:/Users/User/Documents/EQR-handbook")
library(quantreg)
library(foreign)
source("R-progs.R")

# Read VaR data set

var_data <- read.csv("data.csv")
var_data <- na.omit(var_data)


# ====================================

# Generate return assuming lower index represents later observation
gen_return <- function(x)
{
  c((x[1:(length(x)-1)] - x[2:length(x)]) / x[2:length(x)], NA)
}
# Generate lag assuming lower index represents later observation
gen_lag <- function(x, k = 1)
{
  c(x[(k+1):length(x)], rep(NA, k))
}


# ====================================

# Convert date string into date
var_data$Date              <- as.Date(var_data$Date)

# Compute daily return
var_data$DJI_Return        <- gen_return(var_data$DJI_Adj_Close)
var_data$DJUSFN_Return     <- gen_return(var_data$DJUSFN_Adj_Close)
var_data$BAC_Return        <- gen_return(var_data$BAC_Adj_Close)
var_data$C_Return          <- gen_return(var_data$C_Adj_Close)
var_data$JPM_Return        <- gen_return(var_data$JPM_Adj_Close)
var_data$WFC_Return        <- gen_return(var_data$WFC_Adj_Close)

# Compute lagged daily return
var_data$DJI_Return_Lag    <- gen_lag(gen_return(var_data$DJI_Adj_Close))
var_data$DJUSFN_Return_Lag <- gen_lag(gen_return(var_data$DJUSFN_Adj_Close))
var_data$BAC_Return_Lag    <- gen_lag(gen_return(var_data$BAC_Adj_Close))
var_data$C_Return_Lag      <- gen_lag(gen_return(var_data$C_Adj_Close))
var_data$JPM_Return_Lag    <- gen_lag(gen_return(var_data$JPM_Adj_Close))
var_data$WFC_Return_Lag    <- gen_lag(gen_return(var_data$WFC_Adj_Close))

sample.index <- "2009-01-01" <= var_data$Date & var_data$Date <= "2015-11-30"


# ====================================

Y	<- var_data$C_Return[sample.index]
X <- cbind(pmax(var_data$BAC_Return, 0),
           -pmin(var_data$BAC_Return, 0),
           pmax(var_data$JPM_Return, 0),
           -pmin(var_data$JPM_Return, 0),
           pmax(var_data$WFC_Return, 0),
           -pmin(var_data$WFC_Return, 0))[sample.index,]

coef.names <- c("Intercept",
                "BAC Return (+)",
                "BAC Return (-)",
                "JPM Return (+)",
                "JPM Return (-)",
                "WFC Return (+)",
                "WFC Return (-)")


# ====================================
# section 1:  General Results

p <- length(coef.names)
alpha	<- .10
subsample.size <- floor(50 + sqrt(length(Y)))
subsample.fraction <- subsample.size / length(Y)

taus 		    <- (1 : 199) / 200
fit.central	<- array(0, c(p, 3, length(taus)))
fit.extreme	<- array(0, c(p, 3, length(taus)))

for (i in 1 : length(taus))
{
  fit	<- rq(Y ~ X, tau = taus[i])
  
  # Compute confidence bound by CLT
  central	<- summary.rq(fit, se = "ker")
  fit.central[,1,i] <- central$coefficients[,1]
  fit.central[,2,i] <- central$coefficients[,1] + qnorm(alpha/2) * central$coefficients[,2]
  fit.central[,3,i] <- central$coefficients[,1] + qnorm(1-alpha/2) * central$coefficients[,2]
  
  # Compute confidence bound by EVT
  extreme	<- summary.rq.extreme(fit, subsample.fraction=subsample.fraction, R=500, method="br", alpha=alpha, spacing=5+p)
  fit.extreme[,1,i]	<- extreme$coefficients[,5]
  fit.extreme[,2,i] <- extreme$coefficients[,3]
  fit.extreme[,3,i] <- extreme$coefficients[,4]
}


# ====================================
# Save Figure 1 (QR and CI for 0 < tau < 1)

postscript("app3fig1.eps", horizontal=F, pointsize=11, width=6.0, height=8.0)
par(mfrow = c(ceiling(p/2),2))

minscale <- apply(fit.extreme[,2,], 1, min)
maxscale <- apply(fit.extreme[,3,], 1, max)
# Use same scale for positive and negative
for (i in seq(2, p, 2))
{
  minscale[i:(i+1)] <- min(minscale[i:(i+1)])
  maxscale[i:(i+1)] <- max(maxscale[i:(i+1)])
}

for (i in c(2 : p, 1))
{
  # Plot extreme regions
  plot(c(0,1), c(minscale[i], maxscale[i]), xlim=c(0,1), type="n", xlab=expression(tau), ylab="Coefficient")
  title(paste(coef.names[i]), cex=.75)
  lines(taus, smooth(fit.extreme[i,3,]), col=4, lwd=1.5, lty=1)
  lines(taus, smooth(fit.extreme[i,2,]), col=4, lwd=1.5, lty=1)
  abline(h=0)
  # Plot estimates and central regions
  lines(taus, fit.central[i,1,],         col=1, lwd=1.5, lty=1)
  lines(taus, smooth(fit.central[i,3,]), col=2, lwd=1.5, lty=2)
  lines(taus, smooth(fit.central[i,2,]), col=2, lwd=1.5, lty=2)
}

plot(c(0,1), c(0,1), type="n", ylab="", xlab="", cex=.75, axes=FALSE, frame.plot=FALSE)
title("", cex=.75)
type.names <- c("QR Coefficient", "Extremal 90% CI ", "Central 90% CI ")
legend(0, .9, legend=type.names, lty=c(1,1,2), col=c(1,4,2), bty="n", lwd=c(1.5,1.5,1.5), cex=1)

dev.off()


# ====================================
# Save Figure 2 (Bias-corrected QR and CI for tau <= .15)

postscript("app3fig2.eps", horizontal=F, pointsize=11, width=6.0, height=8.0)
par(mfrow = c(ceiling(p/2),2))

minscale <- apply(fit.extreme[, 2, taus <= .16], 1, min)
maxscale <- apply(fit.extreme[, 3, taus <= .16], 1, max)
# Use same scale for positive and negative
for (i in seq(2, p, 2))
{
  minscale[i:(i+1)] <- min(minscale[i:(i+1)])
  maxscale[i:(i+1)] <- max(maxscale[i:(i+1)])
}

for (i in c(2 : p, 1))
{
  # Plot estimates and extreme regions
  plot(range(0,.16), c(minscale[i], maxscale[i]), xlim=c(0,.16), type="n", xlab=expression(tau), ylab="Coefficient")
  title(paste(coef.names[i]), cex=.75)
  lines(taus, smooth(fit.extreme[i,1,]), col=1, lwd=1.5)
  lines(taus, smooth(fit.extreme[i,3,]), col=4, lwd=1.5, lty=1)
  lines(taus, smooth(fit.extreme[i,2,]), col=4, lwd=1.5, lty=1)
  abline(h=0)
  # Plot central regions
  lines(taus, smooth(fit.central[i,3,]), col=2, lwd=1.5, lty=2)
  lines(taus, smooth(fit.central[i,2,]), col=2, lwd=1.5, lty=2)
}

plot(c(0,1), c(0,1), type="n", ylab="", xlab="", cex=.75, axes=FALSE, frame.plot=FALSE)
title("", cex=.75)
type.names <- c("QR Coefficient - BC", "Extremal 90% CI ", "Central 90% CI ")
legend(0, 1, legend=type.names, lty=c(1,1,2), col=c(1,4,2), bty="n", lwd=1.5, cex=1)

dev.off()


# ====================================
# Save Figure 3 (Bias-corrected QR and CI for tau <= .85)

postscript("app3fig3.eps", horizontal=F, pointsize=11, width=6.0, height=8.0)
par(mfrow = c(ceiling(p/2),2))

minscale <- apply(fit.extreme[, 2, taus >= .84], 1, min)
maxscale <- apply(fit.extreme[, 3, taus >= .84], 1, max)
# Use same scale for positive and negative
for (i in seq(2, p, 2))
{
  minscale[i:(i+1)] <- min(minscale[i:(i+1)])
  maxscale[i:(i+1)] <- max(maxscale[i:(i+1)])
}

for (i in c(2 : p, 1))
{
  # Plot estimates and extreme regions
  plot(range(0.84,1), c(minscale[i], maxscale[i]), xlim=c(0.84,1), type="n", xlab=expression(tau), ylab="Coefficient")
  title(paste(coef.names[i]), cex=.75)
  lines(taus, smooth(fit.extreme[i,1,]), col=1, lwd=1.5)
  lines(taus, smooth(fit.extreme[i,3,]), col=4, lwd=1.5, lty=1)
  lines(taus, smooth(fit.extreme[i,2,]), col=4, lwd=1.5, lty=1)
  abline(h=0)
  # Plot central regions
  lines(taus, smooth(fit.central[i,3,]), col=2, lwd=1.5, lty=2)
  lines(taus, smooth(fit.central[i,2,]), col=2, lwd=1.5, lty=2)
}

plot(c(0,1), c(0,1), type="n", ylab="", xlab="", cex=.75, axes=FALSE, frame.plot=FALSE)
title("", cex=.75)
type.names <- c("QR Coefficient - BC", "Extremal 90% CI ", "Central 90% CI ")
legend(0, 1, legend=type.names, lty=c(1,1,2), col=c(1,4,2), bty="n", lwd=1.5, cex=1)

dev.off()



# ====================================
# Compute EV index by Pickands method

set.seed(0)
xi.hat.1 <- summary.rq.hill(rq(Y ~ X, tau=0.01), R=500)
set.seed(0)
xi.hat.2 <- summary.rq.hill(rq(Y ~ X, tau=0.05), R=500)
set.seed(0)
xi.hat.3 <- summary.rq.hill(rq(Y ~ X, tau=0.1), R=500)
EV <- rbind(xi.hat.1, xi.hat.2, xi.hat.3)
round(EV, digits=3)


# ====================================
# Extrapolation estimator

xi.hat <- xi.hat.1[1]
tau.e <- c(0.005, 0.001, 0.0001)
tau.s <- 0.05

beta.sim <- summary.rq(rq(Y ~ X, tau=tau.e[1]), se="ker")$coefficients[,1]
beta.ext.1 <- extrap.rq(Y ~ X, tau.e=tau.e[1], tau.s=tau.s, xi.hat=xi.hat)
beta.ext.2 <- extrap.rq(Y ~ X, tau.e=tau.e[2], tau.s=tau.s, xi.hat=xi.hat)
beta.ext.3 <- extrap.rq(Y ~ X, tau.e=tau.e[3], tau.s=tau.s, xi.hat=xi.hat)
EX <- cbind(beta.sim, beta.ext.1, beta.ext.2, beta.ext.3)
EX <- rbind(c(tau.e[1], tau.e), EX)
colnames(EX) <- c("Regres", "Extrap", "Extrap", "Extrap")
rownames(EX) <- c("tau", coef.names)
round(EX, digits=3)


# ====================================
# Extrapolation fit figure

simple.fit <- cbind(1,X) %*% beta.sim
extrap.fit <- cbind(1,X) %*% beta.ext.1

n <- length(extrap.fit);

postscript("app3fig4.eps", width=8, height=5, horizontal=FALSE, onefile=FALSE, family="AvantGarde")
plot(var_data$Date[3:106], extrap.fit[3:106], type="l", lty=2, xlab="2015", ylab="Conditional 0.005-Quantile")
lines(var_data$Date[3:106], simple.fit[3:106], lty=1)
legend("bottomright", c("Standard fit", "Extrapolated fit"), lty=c(1,2))
axis(1, at=c(22,43,63,84,106), format(var_data$Date[c(22,43,63,84,106)], "%b %Y"))
dev.off()
